/*
 *      L I S T _ D e s t r o y
 *
 *  This function is called when it is required
 *  to destroy a list.
 *
 *
 * (c) Copyright 1996-1999 David J. Walker
 *     Email:  dave.walker@icl.com
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  18 May 96   DJW   - First version.
 *
 *	16 May 99	DJW   -	Amended to use the new LIST_Clear() method to do the
 *						work of clearing nodes from the list.
 */

#include <liblist.h>
#include "liblistx.h"

extern
int		_LIST_Destroy (list_t *plist)
/*		~~~~~~~~~~~~~*/
{
	int		result;

    result = LIST_Clear (*plist);
	if (LIST_ERROR_NONE == result)
	{
		_LIST_UnClone (*plist);
		*plist = NULL;
	}
    return result;
}

extern
int		LIST_Destroy  (list_t * plist)
/*		~~~~~~~~~~~~*/
{
	/*
	 *	If NULL values are passed in we
	 *	simply assume that this routine
	 *	has accidentally been called twice.
	 */
	if (plist && *plist == NULL)
	{
		return LIST_ERROR_NONE;
	}
#ifdef LISTTRACK
    if (_LIST_TrackDelete (*plist) != LIST_ERROR_NONE)
    {
        return LIST_ERROR_NOTFOUND;
    }
#endif /* LISTTRACK */
    LIBDEBUG ("LIST_Destroy", \
           "list does not point to a valid list instance", \
           _LIST_ListCheck(*plist))

	return _LIST_Destroy(plist);
}
