/*
 *      L I S T _ D u p l i c a t e s
 *
 *  This function is called when it is required
 *  to find out if duplicates are allowed or to
 *	change the rules about when they are allowed.
 *
 *
 *  Returns:
 *      -ve		Error code
 *      other   Current setting.  List of valid values is
 *				defined in liblist.h header file.
 *
 *
 * (c) Copyright 1999 David J. Walker
 *     Email:  dave.walker@icl.com
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  14 May 99   DJW   - First version
 */


#include <liblist.h>
#include "liblistx.h"


int     LIST_Duplicates (list_t list, int flags)
{
    register listclass_t listclass;

    LIBDEBUG ("LIST_Duplicates", \
           "'list' parameter is not a valid list instance", \
           _LIST_ListCheck(list))

    listclass = list->listclass;

	switch (flags)
	{
	case LIST_DUPLICATES_QUERY:
					return listclass->duplicates;

	case LIST_DUPLICATES_NA:
			if (listclass->compare != NULL)
			{
					return LIST_ERROR_ORDERED;
			}
			LIBASSERT (listclass->duplicates == LIST_DUPLICATES_NA);
			return LIST_DUPLICATES_NA;

	case LIST_DUPLICATES_ALLOW:
	case LIST_DUPLICATES_BEFORE:
	case LIST_DUPLICATES_AFTER:
			switch (listclass->master->listtype)
			{
			case LIST_TYPE_FIFO:
			case LIST_TYPE_LIFO:
						return LIST_ERROR_UNORDERED;

			case LIST_TYPE_SINGLE:
			case LIST_TYPE_DOUBLE:
			case LIST_TYPE_BTREE:
			case LIST_TYPE_BALTREE:
						if (listclass->compare == NULL)
						{
							return LIST_ERROR_UNORDERED;
						}
						/* FALL THRU */
			case LIST_TYPE_HASH:
						if (LIST_Count(list) != 0)
						{
							return LIST_ERROR_NOTEMPTY;
						}
						listclass->duplicates = flags;
						return flags;
			}

	default:
			return LIST_ERROR_BADTYPE;
	}
}

