/*
 *      L I S T _ E n u m e r a t e
 *
 *  This function is called when it is required
 *  to enumerate a list.
 *
 *  If the list is unordered, then this is simply
 *  a walk through the list in order.
 *
 *  Returns:
 *      -ve     Error occurred
 *      0       OK
 *      +ve     Result from user function
 *
 *
 * (c) Copyright 1996-1999 David J. Walker
 *     Email:  dave.walker@icl.com
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  18 May 96   DJW   - First version
 *
 *	10 July 96	DJW   - Change to handle embedded lists correctly.
 */


#include <liblist.h>
#include "liblistx.h"


long   LIST_Enumerate (list_t list, long (*enumfunc)(node_t, va_list),...)
{
    register node_t  node;
    register _listmaster_t listmaster;

    LIBDEBUG ("LIST_Enumerate", \
           "'list' parameter is not a valid list instance", \
           _LIST_ListCheck(list))
    LIBDEBUG ("LIST_Enumerate", \
           "the node enumeration function parameter is NULL", \
           enumfunc !=  NULL)

	if (LIST_Embedded(list))
	{
		list = ((_embedh_t)list)->self;
	}

    listmaster = list->listclass->master;
    for (node = listmaster->first(list) ; node != NULL ; 
	             node = listmaster->next(list,_LIST_REALNODE(list,node)))
    {
        va_list ap;
        long    result;

        va_start(ap, enumfunc);
        result = enumfunc(node,ap);
        va_end(ap);
        if (result != LIST_ERROR_NONE)
        {
            return result;
        }
    }
    return LIST_ERROR_NONE;
}

