/*
 *      L I S T _ F r e e N o d e
 *
 *  This function is called when it is required
 *  to destroy a node that is not part of a list.
 *
 *
 * (c) Copyright 1996-1999 David J. Walker
 *     Email:  dave.walker@icl.com
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  18 May 96   DJW   - First version
 *
 *	10 July 96	DJW   - Changes to handle embedded lists correctly.
 */

#include <liblist.h>
#include "liblistx.h"
#include <stdlib.h>
#include <string.h>


int     LIST_FreeNode  (list_t list,  node_t * pnode)
{
    listclass_t  listclass;
	node_t	realnode;

	if (! _LIST_ListCheck(list))
	{
		return LIST_ERROR_BADLIST;
	}
	if (*pnode == NULL)
	{
		return LIST_ERROR_BADNODE;
	}

    LIBDEBUG ("LIST_FreeNode", \
            "'node' parameter is not a valid member of this list type",\
            _LIST_NodeCheck(list,*pnode))
    LIBDEBUG ("LIST_FreeNode", \
            "'node is still a member of this list", \
             LIST_Index(list,*pnode) == 0)

	if (LIST_Embedded(list))
	{
		list = _LIST_EmbedOuter(list)->self;
	}

    listclass = list->listclass;

    if (listclass->killnode != NULL)
    {
        int reply;

        reply = listclass->killnode(*pnode);
        if (reply != LIST_ERROR_NONE)
        {
            return (reply);
        }
    }
    realnode =_LIST_REALNODE(list,*pnode);
#ifdef LISTDEBUG
    (void)memset (realnode, 0, listclass->sizenode + listclass->sizeuser);
#endif /* LISTDEBUG */
    free (realnode);
	*pnode = NULL;
    return (LIST_ERROR_NONE);
}
