/*
 *          _ L I S T _ L i f o A f t e r
 *
 *  This routine adds a node on a list that
 *  DOES NOT maintain a pointer to the last node
 *
 *	NOTES.
 *		1.	The node addresses are REAL ones and
 *			not user based ones.
 *		2.	If the 'oldnode' parameter is NULL
 *			then the insert happens at the start
 *			of the list.
 *
 *
 * (c) Copyright 1996-1999 David J. Walker
 *     Email:  dave.walker@icl.com
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  23 May 96   DJW   - First version
 *
 *	09 May 99	DJW	  -	Used casts to keep code clear without need to
 *						assign parameters to local variables.
 */


#include <liblist.h>
#include "liblistx.h"


int     _LIST_LifoAfter (list_t list, node_t newnode, node_t oldnode)
{
/*
 *	When LIBTEST is defined assign the parameters
 *	to local variables to help with debugging.
 */
#ifdef LIBTEST
	_lifoh_t lifolist = (_lifoh_t)list;
	_lifo_t  lifonewnode = (_lifo_t)newnode;
	_lifo_t  lifooldnode = (_lifo_t)oldnode;
#else
#define	lifolist	((_lifoh_t)list)
#define lifonewnode ((_lifo_t)newnode)
#define lifooldnode ((_lifo_t)oldnode)
#endif

    LIBASSERT (newnode != NULL)
    LIBASSERT (lifonewnode->next == NULL)

    /*
     * Empty list ?
     */
    if (lifolist->nodes == NULL)
    {
        lifolist->nodes = lifonewnode;
    }
    else
    {

        if (lifooldnode == NULL)
        {
            /*
             *  At start of existing list
             */
            lifonewnode->next = lifolist->nodes;
            lifolist->nodes = lifonewnode;
        }
        else
        {
            /*
             *  Into middle of existing list
             */

            lifonewnode->next = lifooldnode->next;
            lifooldnode->next = lifonewnode;
        }
    }
    return (LIST_ERROR_NONE);
}
