/*
 *      _ L I S T _ L i s t A d d
 *
 *  This is an internal support routine
 *  to insert a node at the correct point in
 *  an ordered list.   Before we get here we
 *  can assume that the list has already been
 *  checked to see if it is ordered.
 *
 *
 * (c) Copyright 1996-1999 David J. Walker
 *     Email:  dave.walker@icl.com
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  18 May 96   DJW   - First version
 *
 *	14 May 99	DJW   -	Updated to do duplicate handling.
 */


#include <liblist.h>
#include "liblistx.h"


extern
int     _LIST_ListAdd (list_t list, node_t newnode)
{
#ifdef LIBTEST
	_list1h_t singlelist = (_list1h_t)list;
#else
#define singlelist ((_list1h_t)list)
#endif
	listclass_t listclass;
    register _list1_t nextnode;
	register _list1_t prevnode;
	register int	reply;
	int		duplicates;


    LIBASSERT (newnode != NULL)
	
	listclass = singlelist->listclass;
	duplicates = listclass->duplicates;
    /*
     *  We need to scan the whole list in sequence
     */
    for (prevnode = NULL, nextnode = singlelist->nodes ; 
				nextnode != NULL ; 
						prevnode = nextnode, nextnode = nextnode->next)
    {
        /*
         *  If the next node would be too far,
         *  then we must insert after the previous
         *  node (if any) in the list.
         */
        reply = listclass->compare (_LIST_USERNODE(singlelist, newnode), 
                                _LIST_USERNODE(singlelist,nextnode));
		/*
		 *	If we have not yet reached the 
		 *	right node go for the next one
		 */
		if (reply > 0)
		{
            continue;
		}
		/*
		 *	If we are about to over-shoot then
		 *	we have reached insertion point
		 */
		if (reply < 0)
		{
			break;
		}
		/*
		 *	If nodes equal and no duplicates allowed
		 *	then return an error code.
		 */
		if (duplicates == LIST_DUPLICATES_NONE)
		{
			return LIST_ERROR_DUPLICATE;
		}
		/*
		 *	If duplicates allowed, but we insert
		 *	after any duplicates then continue
		 */
		if (duplicates == LIST_DUPLICATES_AFTER)
		{
			continue;
		}
		LIBASSERT (duplicates == LIST_DUPLICATES_BEFORE || LIST_DUPLICATES_ALLOW)
		break;
    }
    return (listclass->master->after((list_t)singlelist, newnode, (node_t)prevnode));
}

