/*
 *      _ L I S T _ L i s t F i n d
 *
 *  This function is an internal support
 *  function called to find a node
 *  on a linked list.
 *
 *
 * (c) Copyright 1996-1999 David J. Walker
 *     Email:  dave.walker@icl.com
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  18 May 96   DJW   - First version
 */


#include <liblist.h>
#include "liblistx.h"


extern
node_t  _LIST_ListFind  (list_t list, 
                         int (*findfunc)(node_t, va_list), 
                         va_list ap)
{
    register _list1_t listnode;
    register node_t usernode;
    register int reply;
    register int ordered;

    /*
     *  Scan the list looking for the node
     */
    for (listnode = (_list1_t)list->nodes, ordered = (list->listclass->compare!=0) ; 
                listnode != NULL ; listnode = listnode->next)
    {
        usernode = _LIST_USERNODE(list,listnode);
        reply = (*findfunc)(usernode,ap);
        switch (reply)
        {
        case 0:
                return (usernode);

        case LIST_ORDER_NA:
                break;

        default:
                    /*
                     *  If the list is ordered and we
                     *  have overshot, we can return
                     *  the failure condition immediately.
                     */
                if (reply > 0  && ordered != 0)
                {
                    return NULL;
                }
                break;
        }
    }
    /*
     *  If we got here it is not found
     *  and we must return NULL value.
     */
    return (NULL);
}


