/*
 *      L I S T _ N e w G e t N o d e
 *
 *  This function is an internal support routine that
 *  is used by all the routines that both allocate a
 *	new node and then add it to the list in the same
 *	call.
 *
 * (c) Copyright 1996-1999 David J. Walker
 *     Email:  dave.walker@icl.com
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  22 May 96   DJW   - First version
 *
 *  03 May 99   DJW   - Corrected problem where absence of an 'initnode'
 *                      routine return LIST_ERROR_ORDERED instead of the
 *                      correct code of LIST_ERROR_BADINIT.
 *
 *	09 May 99	DJW	  -	Moved code shared by many of the New'oper' family of routines
 *						into this support routine.
 *						Fixed problem where error code could be corrupted when operation failed.
 */

#include <liblist.h>
#include "liblistx.h"


/*
 *  This function combines the LIST_New() functionality
 *  with that of the LIST_Insert() method in one call.
 */
node_t    _LIST_NewGetNode (list_t list, va_list ap)
{
    listclass_t listclass;
    node_t  newnode;

    LIBDEBUG ("_LIST_NewGetNode", \
           "'list' parameter is not a valid list instance", \
           _LIST_ListCheck(list))

	listclass = list->listclass;

    /*
     *  Check that a node initialisation
     *  routine has been provided.
     */
    if (listclass->initnode == NULL)
    {
        errno = LIST_ERROR_BADINIT;
        return NULL;
    }
    /*
     *  Create a new node
     *  (passing on variable parameter list)
     */
    newnode = listclass->master->nodenew(list, ap);
	LIBASSERT(_LIST_NodeCheck(list,newnode));

    return (newnode);
}

