/*
 *      L I S T _ N e w P u s h
 *
 *  This function is called when it is required
 *  to create a new node and insert it at the 
 *  beginning of a list being treated as a stack.
 *
 *
 * (c) Copyright 1996-1999 David J. Walker
 *     Email:  dave.walker@icl.com
 *
 *     Permission to copy and/or distribute granted under the
 *     following conditions:
 *
 *     1). This notice must remain intact.
 *     2). The author is not responsible for the consequences of use
 *         this software, no matter how awful, even if they
 *         arise from defects in it.
 *     3). Altered version must not be represented as being the
 *         original software.
 *
 *
 *  AMENDMENT HISTORY
 *  ~~~~~~~~~~~~~~~~~
 *  22 May 96   DJW   - First version
 *
 *	09 May 99	DJW	  -	Changed to use new support routine _LIST_NewGetNode
 *						Fixed problem where error code could be corrupted when operation failed.
 */


#include <liblist.h>
#include "liblistx.h"


/*
 *  This function combines the LIST_New() functionality
 *  with that of the LIST_Push() method in one call.
 */
node_t    LIST_NewPush (list_t list, ...)
{
    node_t  newnode;
    int     reply;
    va_list ap;

    LIBDEBUG ("LIST_NewPush", \
            "'list' parameter is not a valid list pointer", \
            _LIST_ListCheck(list))

    /*
     *  Create a new node
     *  (passing on variable parameter list)
     */
    va_start (ap, list);
    newnode = _LIST_NewGetNode(list, ap);
    va_end(ap);
    if (newnode != NULL)
    {
	    reply = LIST_Push(list,newnode);
		if (reply != LIST_ERROR_NONE)
	    {
#ifdef LIBTEST
		    LIBASSERT (LIST_FreeNode(list,&newnode) == LIST_ERROR_NONE)
#else
			(void)LIST_FreeNode(list,&newnode);
#endif
	        errno = reply;
		    newnode = NULL;
		}
	}
    return (newnode);
}


